@@ -30,7 +30,7 @@ static int
 is_hex(const char *p, size_t l)
 {
 	while (l > 0) {
-		if (*p >= 0 && *p <= '9') {
+		if (*p >= '0' && *p <= '9') {
 			/* Ascii digit */
 		} else if (*p >= 'a' && *p <= 'f') {
 			/* lowercase letter a-f */
@@ -61,7 +61,7 @@ DEFINE_TEST(test_write_format_cpio_newc)
 	/* Create a new archive in memory. */
 	assert((a = archive_write_new()) != NULL);
 	assertEqualIntA(a, 0, archive_write_set_format_cpio_newc(a));
-	assertEqualIntA(a, 0, archive_write_set_compression_none(a));
+	assertEqualIntA(a, 0, archive_write_add_filter_none(a));
 	assertEqualIntA(a, 0, archive_write_open_memory(a, buff, buffsize, &used));
 
 	/*
